package org.jeecg.modules.redis;

import static org.springframework.data.redis.serializer.RedisSerializationContext.SerializationPair;

import lombok.extern.slf4j.Slf4j;

import org.apache.commons.lang3.StringUtils;
import org.jeecg.modules.adweb.dmp.service.impl.GACountryReportServiceImpl;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.cache.*;
import org.springframework.data.redis.serializer.GenericJackson2JsonRedisSerializer;

import java.time.Duration;
import java.util.Date;

/**
 * 支持在{@link Cacheable#cacheNames()}中设置Redis TTL，以#分隔，单位为秒
 *
 * <p>示例: @Cacheable(cacheManager = CacheConfig.TTL_CACHE_MANAGER, cacheNames = "test:demo#3600")
 *
 * <p>用法参考{@link GACountryReportServiceImpl#getCountryStats(String, Date, Date)}方法
 *
 * @author wfansh
 */
@Slf4j
public class TTLCacheManager extends RedisCacheManager {

    public TTLCacheManager(
            RedisCacheWriter cacheWriter, RedisCacheConfiguration defaultCacheConfiguration) {
        super(cacheWriter, defaultCacheConfiguration);
    }

    @Override
    protected RedisCache createRedisCache(String name, RedisCacheConfiguration cacheConfiguration) {
        // 修改缓存key和value值的序列方式
        cacheConfiguration =
                cacheConfiguration
                        .computePrefixWith(cacheName -> cacheName + ":")
                        .serializeValuesWith(
                                SerializationPair.fromSerializer(
                                        new GenericJackson2JsonRedisSerializer()));

        final int idx = StringUtils.lastIndexOf(name, '#');
        if (idx > -1) {
            final String ttl = StringUtils.substring(name, idx + 1);
            name = StringUtils.substring(name, 0, idx);

            // 设置TTL
            cacheConfiguration =
                    cacheConfiguration.entryTtl(Duration.ofSeconds(Long.parseLong(ttl)));
        }

        return super.createRedisCache(name, cacheConfiguration);
    }
}
